/*!
 * \brief Argument parser for the demo application
 * \copyright @ 2022 RnD Center ELVEES, JSC
 */

#ifndef ARG_PARSER_H
#define ARG_PARSER_H

#include <string>
#include <memory>
#include <vector>

#include <pipeline_configs.h>

class ArgsParser {
  public:
  static ArgsParser* Parse(int argc, char** argv);

  std::string GetInputPipelineString() const;
  std::string GetOutputPipelineString() const;
  uint32_t GetDSPId() const;
  const std::string& GetExamplesDir() const;

  private:
  std::shared_ptr<Config> input_cfg = nullptr;
  std::shared_ptr<Config> output_cfg = nullptr;
  uint32_t dsp_id = 0;
  std::string examples_dir =
      std::string(getenv("HOME")) + "/elcorenn-examples-data";

  ArgsParser(){};

  static const std::string GetHelpMessage();
  bool ParseInput(std::string str, std::string framerate);
  bool ParseOutput(std::string str);
};

#endif // ARG_PARSER_H
