/*!
 * \brief Set of the gstreamer pipeline configs for the demo application
 * \copyright @ 2022 RnD Center ELVEES, JSC
 */

#ifndef PIPELINE_CONFIGS_H
#define PIPELINE_CONFIGS_H

#include <string>

class Config {
  public:
  virtual ~Config() {
  }
  virtual std::string Get() = 0;
};

class InputConfig : public Config {
  public:
  InputConfig(const std::string& path, const std::string& framerate) :
      path(path), framerate(framerate) {
  }

  std::string Get() final {
    if (framerate.empty()) {
      return "filesrc location=" + path + " " +
             "! h264parse "
             "! omxh264dec ! video/x-raw,format=NV12";
    } else {
      return "filesrc location=" + path + " " +
             "! h264parse "
             "! omxh264dec ! video/x-raw,format=NV12 "
             "! videorate  ! video/x-raw,framerate=" +
             framerate;
    }
  }

  private:
  std::string path;
  std::string framerate;
};

class OutputConfigHdmiMCom03 : public Config {
  public:
  std::string Get() final {
    return "kmssink driver-name=mali-dp";
  }
};

class OutputConfigHdmiSolaris : public Config {
  public:
  std::string Get() final {
    return "videoconvert ! video/x-raw, format=NV12 "
           "! omxh264enc ! h264parse ! video/x-h264 "
           "! kmssink connector-id=42 driver-name=pdp-drm";
  }
};

class OutputConfigRtsp : public Config {
  public:
  OutputConfigRtsp(const std::string& address) : address(address) {
  }

  std::string Get() final {
    return "videoconvert ! video/x-raw, format=NV12 "
           "! omxh264enc ! h264parse ! video/x-h264 "
           "! rtspclientsink location=" +
           address;
  }

  private:
  std::string address;
};

class OutputConfigFile : public Config {
  public:
  OutputConfigFile(const std::string& file_path) : file_path(file_path) {
  }

  std::string Get() final {
    return "videoconvert ! video/x-raw, format=NV12 "
           "! omxh264enc ! filesink location=" +
           file_path;
  }

  private:
  std::string file_path;
};

#endif // PIPELINE_CONFIGS_H
