// Simple example of full-connected network.
// It takes a number and multiplies it by 1.

#include <iostream>
#include <vector>

#include <elcorecl/elcorecl.h>
#include <elcorenn/elcorenn.h>

const auto batchSize = 32;

void print_usage(const std::string& name) {
  std::cout << "Usage: " << name << " [OPTIONS]" << std::endl
            << "Options description:" << std::endl
            << "    -d   Path to elcorenn-examples-data" << std::endl
            << "         Default: $HOME/elcorenn-examples-data" << std::endl
            << "    -h   Print usage." << std::endl;
}

int main(int argc, char** argv) {
  std::string home_dir = getenv("HOME");
  std::string examples_dir = home_dir + "/elcorenn-examples-data";

  for (int i = 1; i < argc; ++i) {
    if (std::string(argv[i]) == "-h") {
      print_usage(argv[0]);
      return 0;
    }
    if (std::string(argv[i]) == "-d") {
      if (i + 1 == argc) {
        std::cout << "Wrong number of arguments!" << std::endl;
        print_usage(argv[0]);
        return -1;
      }
      examples_dir = std::string(argv[i + 1]);
    } else {
      std::cout << "Unrecognized argument: " << std::string(argv[i])
                << std::endl;
      print_usage(argv[0]);
      return -2;
    }
  }

  std::string model_json = examples_dir + "/models/fc_model.json";
  std::string model_weights = examples_dir + "/models/fc_weights.bin";

  // Get DSPs count
  ecl_platform_id platform_id;
  ecl_int ret = eclGetPlatformIDs(1, &platform_id, nullptr);
  ecl_uint support_dev_count = 0;
  ret = eclGetDeviceIDs(platform_id, ECL_DEVICE_TYPE_CUSTOM, 0, nullptr,
                        &support_dev_count);

  // Init ElcoreNN resources
  auto backend_id = InitBackend(support_dev_count);

  std::vector<float> inputData;
  std::vector<float> outputData(batchSize);

  // Input data preparation
  inputData.reserve(batchSize);
  for (auto i = 0; i < batchSize; i++) {
    inputData.push_back(static_cast<float>(i) + 1.0f);
  }

  // Load model from files
  auto model_id = LoadModel(model_json.c_str(), model_weights.c_str(),
                            ENNDataType::FLOAT16, backend_id);

  // Вычисление сети
  float* inputs[1] = {inputData.data()};
  float* outputs[1] = {outputData.data()};

  InvokeModel(model_id, inputs, outputs, batchSize);

  // Print results
  std::cout << "Input:" << std::endl;
  for (auto i : inputData) {
    std::cout << " " << i;
  }
  std::cout << std::endl;

  std::cout << "Output: " << std::endl;
  for (auto i : outputData) {
    std::cout << " " << i;
  }
  std::cout << std::endl;

  ReleaseModel(model_id);
  ReleaseBackend(backend_id);

  return 0;
}
