#pragma once

#include <iostream>
#include <vector>

#include <opencv2/core.hpp>
#include <opencv2/imgcodecs.hpp>
#include <opencv2/imgproc.hpp>
#include <elcorenn/elcorenn.h>

#include "coco_classes.hpp"
#include "nms.hpp"

struct Detection {
  int class_id;
  float confidence;
  cv::Rect box;
};

void output_proc(const cv::Mat& input_image, std::vector<Detection>& preds,
                 const std::vector<std::pair<std::string, cv::Scalar>>& classes,
                 const double score_threshold, const double nms_threshold,
                 const double confidence_threshold) {
  float* data = reinterpret_cast<float*>(input_image.data);

  const int dimensions = 85;
  const int rows = 25200;

  std::vector<int> class_ids;
  std::vector<float> confidences;
  std::vector<cv::Rect> boxes;

  for (int i = 0, k = 0; i < rows; ++i) {
    float confidence = data[4];

    if (confidence >= confidence_threshold) {
      float* classes_scores = data + 5;
      cv::Mat scores(1, classes.size(), CV_32FC1, classes_scores);

      int max_idx = 0;
      for (int i = 1; i < scores.size().width; ++i) {
        if (scores.at<float>(0, i) > scores.at<float>(0, max_idx))
          max_idx = i;
      }

      if (scores.at<float>(0, max_idx) > score_threshold) {
        confidences.push_back(confidence);
        class_ids.push_back(max_idx);

        float x = data[0];
        float y = data[1];
        float w = data[2];
        float h = data[3];
        int left = x - 0.5 * w;
        int top = y - 0.5 * h;
        boxes.push_back(cv::Rect(left, top, w, h));
      }
    }
    data += dimensions;
  }

  std::vector<int> nms_result;
  NMSBoxes(boxes, confidences, score_threshold, nms_threshold, nms_result);

  for (int i = 0; i < nms_result.size(); i++) {
    int idx = nms_result[i];
    Detection result;
    result.class_id = class_ids[idx];
    result.confidence = confidences[idx];
    result.box = boxes[idx];
    preds.push_back(result);
  }
}
